////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2014 The Foundry Visionmongers Ltd.  All Rights Reserved.
////////////////////////////////////////////////////////////////////////////////

// This file contains code which is common to mov64Reader & mov64Writer.

#ifndef NUKE_MOV64COMMON_H
#define NUKE_MOV64COMMON_H

// String array returned by EnumerationKnobI::menu. Here so mov64 reader/writer
// can include movCommon.
typedef std::vector<std::string> StringArray;

////////////////////////////////////////////////////////////////////////////////
// NCLC static names and mappings.
////////////////////////////////////////////////////////////////////////////////

//Used in ffmpeg metadata dicts to stash values written to and from nclc atom.
static const char kNCLCPrimariesKey[]   = "fn_primaries";
static const char kNCLCTransferKey[]    = "fn_transfer_function";
static const char kNCLCMatrixKey[]      = "fn_matrix";

//Labels used in a batch of places.
static const char kNCLCUnknownLabel[]   = "Unknown";
static const char kNCLCReservedLabel[]  = "Reserved";

// Avid DNxHD specific. Label for switching between video legal and full range.
static const char kARESCompressionID[]  = "fn_ares_compression_id";
static const char kACLRYuvRange[]       = "fn_aclr_yuv_range";

// Get the display string for a COLR/NCLC atom's primaries, based on index.
//
// param index:     NCLC primaries index
static const std::string GetNCLCPrimariesLabelFromIndex(const int index)
{
  switch (index) {
    case 1:
    case 6:
      return std::string("ITU_R_709_2");
      break;
    case 2:
      return std::string(kNCLCUnknownLabel);
      break;
    case 5:
      return std::string("EBU_3213");
      break;
    default:
      return std::string(kNCLCReservedLabel);
      break;
  }
}

// Get the display string for a COLR/NCLC atom's transfer func, based on index.
//
// param index:     NCLC transfer index
static const std::string GetNCLCTransferLabelFromIndex(const int index)
{
  switch (index) {
    case 1:
      return std::string("ITU_R_709_2");
      break;
    case 2:
      return std::string(kNCLCUnknownLabel);
      break;
    case 7:
      return std::string("SMPTE_240M");
      break;
    default:
      return std::string(kNCLCReservedLabel);
      break;
  }
}

// Get the display string for a COLR/NCLC atom's YCbCr matrix, based on index.
//
// param index:     NCLC matrix index
static const std::string GetNCLCMatrixLabelFromIndex(const int index)
{
  switch (index) {
    case 1:
      return std::string("ITU_R_709_2");
      break;
    case 2:
      return std::string(kNCLCUnknownLabel);
      break;
    case 6:
      return std::string("ITU_R_601_4");
    case 7:
      return std::string("SMPTE_240M");
      break;
    default:
      return std::string(kNCLCReservedLabel);
      break;
  }
}

////////////////////////////////////////////////////////////////////////////////
// Chunksize static names.
////////////////////////////////////////////////////////////////////////////////

static const char kChunkSizeKey[]      = "fn_log2chunksize";

#endif // NUKE_MOV64COMMON_H
